from configparser import ConfigParser
from cat_win.const.ColorConstants import ColorOptions, C_KW


class Config:
    default_dic = {C_KW.NUMBER: ColorOptions.Fore['GREEN'],
                   C_KW.LINE_LENGTH: ColorOptions.Fore['LIGHTBLUE'],
                   C_KW.ENDS: ColorOptions.Back['YELLOW'],
                   C_KW.TABS: ColorOptions.Back['YELLOW'],
                   C_KW.CONVERSION: ColorOptions.Fore['CYAN'],
                   C_KW.REPLACE: ColorOptions.Fore['YELLOW'],
                   C_KW.FOUND: ColorOptions.Fore['RED'],
                   C_KW.FOUND_MESSAGE: ColorOptions.Fore['MAGENTA'],
                   C_KW.MATCHED: ColorOptions.Back['CYAN'],
                   C_KW.MATCHED_MESSAGE: ColorOptions.Fore['LIGHTCYAN'],
                   C_KW.CHECKSUM: ColorOptions.Fore['CYAN'],
                   C_KW.COUNT_AND_FILES: ColorOptions.Fore['CYAN'],
                   C_KW.ATTRIB_POSITIVE: ColorOptions.Fore['LIGHTGREEN'],
                   C_KW.ATTRIB_NEGATIVE: ColorOptions.Fore['LIGHTRED'],
                   C_KW.ATTRIB: ColorOptions.Fore['CYAN']}
    elements = list(default_dic.keys())

    def __init__(self, workingDir) -> None:
        self.workingDir = workingDir
        self.configFile = self.workingDir + "/cat.config"
        
        self.exclusive_definitions = {'Fore': [C_KW.FOUND],  # can only be Foreground
                                      'Back': [C_KW.MATCHED]}  # can only be Background
        self.configParser = ConfigParser()
        self.color_dic = {}

    def loadConfig(self) -> dict:
        """
        Load the Color Configuration from the config file and return it.
        On Error: Return the default color config.
        """
        try:
            self.configParser.read(self.configFile)
            configColors = self.configParser['COLORS']
            for element in self.elements:
                try:
                    type, color = configColors[element].split(".")
                    self.color_dic[element] = (
                        ColorOptions.Fore[color] if type == 'Fore' else ColorOptions.Back[color])
                except KeyError:
                    self.color_dic[element] = self.default_dic[element]
        except KeyError:
            self.configParser['COLORS'] = {}
            # If an error occures we simplfy use the default colors
            self.color_dic = self.default_dic

        # The Reset Codes should always be the same
        self.color_dic[C_KW.RESET_ALL] = ColorOptions.Style['RESET']
        self.color_dic[C_KW.RESET_FOUND] = ColorOptions.Fore['RESET']
        self.color_dic[C_KW.RESET_MATCHED] = ColorOptions.Back['RESET']

        return self.color_dic

    def _printGetAllAvailableColors(self) -> list:
        options = []
        index = 1
        print("Here is a list of all available color options you may choose:")
        for key, value in ColorOptions.Fore.items():
            if key == 'RESET':
                continue
            option = "Fore." + key
            print(f"{index: <2}: ", value, option,
                  ColorOptions.Fore['RESET'], sep="")
            options.append(option)
            index += 1
        for key, value in ColorOptions.Back.items():
            if key == 'RESET':
                continue
            option = "Back." + key
            print(f"{index: <2}: ", value, option,
                  ColorOptions.Back['RESET'], sep="")
            options.append(option)
            index += 1
        return options

    def _printAllAvailableElements(self) -> None:
        print("Here is a list of all available elements you may change:")
        for index, element in enumerate(self.elements, start=1):
            print(f"{index: <2}: ", self.color_dic[element], element,
                  ColorOptions.Style['RESET'], sep="")

    def saveConfig(self) -> None:
        """
        Guide the User through the configuration options and save the changes.
        Assume, that the current config is already loaded/the method loadConfig() was already called.
        """
        self._printAllAvailableElements()
        keyword = ''
        while (not keyword in self.elements):
            if keyword != '':
                print(f"Something went wrong. Unknown keyword '{keyword}'")
            keyword = input("Input name of keyword to change: ")
            if keyword.isdigit():
                keyword = self.elements[int(keyword)-1] if (
                    0 < int(keyword) <= len(self.elements)) else keyword
        print(f"Successfully selected element '{keyword}'.")

        color_options = self._printGetAllAvailableColors()
        color = ''
        while (not color in color_options):
            if color != '':
                print(f"Something went wrong. Unknown option '{color}'.")
            color = input("Input color: ")
            if color.isdigit():
                color = color_options[int(color)-1] if (
                    0 < int(color) <= len(color_options)) else color

        if keyword in self.exclusive_definitions['Fore'] and color.startswith('Back'):
            print(f"An Error occured: '{keyword}' can only be of style 'Fore'")
            return
        if keyword in self.exclusive_definitions['Back'] and color.startswith('Fore'):
            print(f"An Error occured: '{keyword}' can only be of style 'Back'")
            return

        print(f"Successfully selected element '{color}'.")

        self.configParser['COLORS'][keyword] = color
        try:
            with open(self.configFile, 'w') as conf:
                self.configParser.write(conf)
            print(f"Successfully updated config file:\n\t{self.configFile}")
        except:
            print(f"Could not write to config file:\n\t{self.configFile}")
